@@ -27,15 +27,14 @@ Vinay Sajip to make use of the subprocess module (Steve's version uses os.fork()
 and so does not work on Windows). Renamed to gnupg.py to avoid confusion with
 the previous versions.
 
-Modifications Copyright (C) 2008-2013 Vinay Sajip. All rights reserved.
+Modifications Copyright (C) 2008-2014 Vinay Sajip. All rights reserved.
 
 A unittest harness (test_gnupg.py) has also been added.
 """
-import locale
 
-__version__ = "0.3.5"
+__version__ = "0.3.6"
 __author__ = "Vinay Sajip"
-__date__  = "$30-Aug-2013 18:10:36$"
+__date__  = "$05-Feb-2014 17:47:10$"
 
 try:
     from io import StringIO
@@ -103,12 +102,10 @@ else:
             raise TypeError('Expected string type, got %s' % type(s))
         if not s:
             result = "''"
-        elif len(s) >= 2 and (s[0], s[-1]) == ("'", "'"):
-            result = '"%s"' % s.replace('"', r'\"')
         elif not UNSAFE.search(s):
             result = s
         else:
-            result = "'%s'" % s.replace("'", "'\"'\"'")
+            result = "'%s'" % s.replace("'", r"'\''")
         return result
 
     # end of sarge code
@@ -153,11 +150,19 @@ def _write_passphrase(stream, passphrase, encoding):
     passphrase = '%s\n' % passphrase
     passphrase = passphrase.encode(encoding)
     stream.write(passphrase)
-    logger.debug("Wrote passphrase: %r", passphrase)
+    logger.debug('Wrote passphrase')
 
 def _is_sequence(instance):
     return isinstance(instance, (list, tuple, set, frozenset))
 
+def _make_memory_stream(s):
+    try:
+        from io import BytesIO
+        rv = BytesIO(s)
+    except ImportError:
+        rv = StringIO(s)
+    return rv
+
 def _make_binary_stream(s, encoding):
     if _py3k:
         if isinstance(s, str):
@@ -165,12 +170,7 @@ def _make_binary_stream(s, encoding):
     else:
         if type(s) is not str:
             s = s.encode(encoding)
-    try:
-        from io import BytesIO
-        rv = BytesIO(s)
-    except ImportError:
-        rv = StringIO(s)
-    return rv
+    return _make_memory_stream(s)
 
 class Verify(object):
     "Handle status messages for --verify"
@@ -680,18 +680,18 @@ class GPG(object):
         will be appended. The ``passphrase`` argument needs to be True if
         a passphrase will be sent to GPG, else False.
         """
-        cmd = [self.gpgbinary, '--status-fd 2 --no-tty']
+        cmd = [self.gpgbinary, '--status-fd', '2', '--no-tty']
         if self.gnupghome:
-            cmd.append('--homedir %s' % shell_quote(self.gnupghome))
+            cmd.extend(['--homedir',  shell_quote(self.gnupghome)])
         if self.keyring:
             cmd.append('--no-default-keyring')
             for fn in self.keyring:
-                cmd.append('--keyring %s' % shell_quote(fn))
+                cmd.extend(['--keyring', shell_quote(fn)])
         if self.secret_keyring:
             for fn in self.secret_keyring:
-                cmd.append('--secret-keyring %s' % shell_quote(fn))
+                cmd.extend(['--secret-keyring', shell_quote(fn)])
         if passphrase:
-            cmd.append('--batch --passphrase-fd 0')
+            cmd.extend(['--batch', '--passphrase-fd', '0'])
         if self.use_agent:
             cmd.append('--use-agent')
         if self.options:
@@ -702,11 +702,16 @@ class GPG(object):
     def _open_subprocess(self, args, passphrase=False):
         # Internal method: open a pipe to a GPG subprocess and return
         # the file objects for communicating with it.
-        cmd = ' '.join(self.make_args(args, passphrase))
+        cmd = self.make_args(args, passphrase)
+        pcmd = ' '.join(cmd)
         if self.verbose:
-            print(cmd)
+            print(pcmd)
         logger.debug("%s", cmd)
-        return Popen(cmd, shell=True, stdin=PIPE, stdout=PIPE, stderr=PIPE)
+        # For some reason not yet established, shell=False fails on Windows.
+        if os.name == 'posix':
+            return Popen(cmd, shell=False, stdin=PIPE, stdout=PIPE, stderr=PIPE)
+        else:
+            return Popen(pcmd, shell=True, stdin=PIPE, stdout=PIPE, stderr=PIPE)
 
     def _read_response(self, stream, result):
         # Internal method: reads all the stderr output from GPG, taking notice
@@ -823,7 +828,7 @@ class GPG(object):
         elif clearsign:
             args.append("--clearsign")
         if keyid:
-            args.append('--default-key %s' % shell_quote(keyid))
+            args.extend(['--default-key', shell_quote(keyid)])
         result = self.result_map['sign'](self)
         #We could use _handle_io here except for the fact that if the
         #passphrase is bad, gpg bails and you can't write the message.
@@ -884,6 +889,15 @@ class GPG(object):
                 os.unlink(fn)
         return result
 
+    def verify_data(self, sig_filename, data):
+        "Verify the signature in sig_filename against data in memory"
+        logger.debug('verify_data: %r, %r ...', sig_filename, data[:16])
+        result = self.result_map['verify'](self)
+        args = ['--verify', shell_quote(sig_filename), '-']
+        stream = _make_memory_stream(data)
+        self._handle_io(args, stream, result, binary=True)
+        return result
+
     #
     # KEY MANAGEMENT
     #
@@ -985,10 +999,11 @@ class GPG(object):
         if secret:
             which='secret-key'
         if _is_sequence(fingerprints):
-            fingerprints = ' '.join([shell_quote(s) for s in fingerprints])
+            fingerprints = [shell_quote(s) for s in fingerprints]
         else:
-            fingerprints = shell_quote(fingerprints)
-        args = ['--batch --delete-%s %s' % (which, fingerprints)]
+            fingerprints = [shell_quote(fingerprints)]
+        args = ['--batch', '--delete-%s' % which]
+        args.extend(fingerprints)
         result = self.result_map['delete'](self)
         p = self._open_subprocess(args)
         self._collect_output(p, result, stdin=p.stdin)
@@ -1000,10 +1015,11 @@ class GPG(object):
         if secret:
             which='-secret-key'
         if _is_sequence(keyids):
-            keyids = ' '.join([shell_quote(k) for k in keyids])
+            keyids = [shell_quote(k) for k in keyids]
         else:
-            keyids = shell_quote(keyids)
-        args = ['--armor --export%s %s' % (which, keyids)]
+            keyids = [shell_quote(keyids)]
+        args = ['--armor', '--export%s' % which]
+        args.extend(keyids)
         p = self._open_subprocess(args)
         # gpg --export produces no status-fd output; stdout will be
         # empty in case of failure
@@ -1033,8 +1049,7 @@ class GPG(object):
         which='keys'
         if secret:
             which='secret-keys'
-        args = "--list-%s --fixed-list-mode --fingerprint --with-colons" % (which,)
-        args = [args]
+        args = ["--list-%s" % which, "--fixed-list-mode", "--fingerprint", "--with-colons"]
         p = self._open_subprocess(args)
 
         # there might be some status thingumy here I should handle... (amk)
@@ -1112,7 +1127,7 @@ class GPG(object):
         >>> assert not result
 
         """
-        args = ["--gen-key --batch"]
+        args = ["--gen-key", "--batch"]
         result = self.result_map['generate'](self)
         f = _make_binary_stream(input, self.encoding)
         self._handle_io(args, f, result, binary=True)
@@ -1186,15 +1201,17 @@ class GPG(object):
             if not _is_sequence(recipients):
                 recipients = (recipients,)
             for recipient in recipients:
-                args.append('--recipient %s' % shell_quote(recipient))
+                args.extend(['--recipient', shell_quote(recipient)])
         if armor:   # create ascii-armored output - False for binary output
             args.append('--armor')
         if output:  # write the output to a file with the specified name
             if os.path.exists(output):
-                os.remove(output) # to avoid overwrite confirmation message
-            args.append('--output %s' % shell_quote(output))
+                # We need to avoid an overwrite confirmation message
+                #os.remove(output)
+                args.extend(['--batch', '--yes'])
+            args.extend(['--output', shell_quote(output)])
         if sign:
-            args.append('--sign --default-key %s' % shell_quote(sign))
+            args.extend(['--sign', '--default-key', shell_quote(sign)])
         if always_trust:
             args.append("--always-trust")
         result = self.result_map['crypt'](self)
@@ -1259,8 +1276,10 @@ class GPG(object):
         args = ["--decrypt"]
         if output:  # write the output to a file with the specified name
             if os.path.exists(output):
-                os.remove(output) # to avoid overwrite confirmation message
-            args.append('--output %s' % shell_quote(output))
+                # We need to avoid an overwrite confirmation message
+                #os.remove(output)
+                args.extend(['--batch', '--yes'])
+            args.extend(['--output', shell_quote(output)])
         if always_trust:
             args.append("--always-trust")
         result = self.result_map['crypt'](self)
