@@ -2,13 +2,14 @@
 """
 A test harness for gnupg.py.
 
-Copyright (C) 2008-2013 Vinay Sajip. All rights reserved.
+Copyright (C) 2008-2014 Vinay Sajip. All rights reserved.
 """
 import doctest
 import logging
 import os.path
 import os
 import shutil
+import stat
 import sys
 import tempfile
 import unittest
@@ -16,7 +17,7 @@ import unittest
 import gnupg
 
 __author__ = "Vinay Sajip"
-__date__  = "$30-Aug-2013 18:10:57$"
+__date__  = "$05-Feb-2014 17:48:48$"
 
 ALL_TESTS = True
 
@@ -98,9 +99,13 @@ class GPGTestCase(unittest.TestCase):
                             "Not a directory: %s" % hd)
             shutil.rmtree(hd)
         self.homedir = hd
-        self.gpg = gnupg.GPG(gnupghome=hd, gpgbinary=GPGBINARY)
-        if self.gpg.version and self.gpg.version >= (2,):
-            self.gpg.options = ['--debug-quick-random']
+        self.gpg = gpg = gnupg.GPG(gnupghome=hd, gpgbinary=GPGBINARY)
+        v = gpg.version
+        if v:
+            if v >= (2,):
+                gpg.options = ['--debug-quick-random']
+            else:
+                gpg.options = ['--quick-random']
 
     def test_environment(self):
         "Test the environment by ensuring that setup worked"
@@ -129,12 +134,14 @@ class GPGTestCase(unittest.TestCase):
             'Name-Comment': 'A test user',
             'Expire-Date': 0,
         }
-        if '--debug-quick-random' in (self.gpg.options or []):
+        options = self.gpg.options or []
+        if '--debug-quick-random' in options or '--quick-random' in options:
             # If using the fake RNG, a key isn't regarded as valid
             # unless its comment has the text (insecure!) in it.
             params['Name-Comment'] = 'A test user (insecure!)'
         params['Name-Real'] = '%s %s' % (first_name, last_name)
-        params['Name-Email'] = ("%s.%s@%s" % (first_name, last_name, domain)).lower()
+        params['Name-Email'] = ("%s.%s@%s" % (first_name, last_name,
+                                              domain)).lower()
         if passphrase is None:
             passphrase = ("%s%s" % (first_name[0], last_name)).lower()
         params['Passphrase'] = passphrase
@@ -346,11 +353,12 @@ class GPGTestCase(unittest.TestCase):
                          "Fingerprints must match")
         self.assertEqual(verified.trust_level, verified.TRUST_ULTIMATE)
         self.assertEqual(verified.trust_text, 'TRUST_ULTIMATE')
-        if not os.path.exists('random_binary_data'):
-            data_file = open('random_binary_data', 'wb')
+        test_fn = 'random_binary_data'
+        if not os.path.exists(test_fn):
+            data_file = open(test_fn, 'wb')
             data_file.write(os.urandom(5120 * 1024))
             data_file.close()
-        data_file = open('random_binary_data', 'rb')
+        data_file = open(test_fn, 'rb')
         sig = self.gpg.sign_file(data_file, keyid=key.fingerprint,
                                  passphrase='aable')
         data_file.close()
@@ -367,7 +375,7 @@ class GPGTestCase(unittest.TestCase):
             logger.debug("ver: %r", verified.fingerprint)
         self.assertEqual(key.fingerprint, verified.fingerprint,
                          "Fingerprints must match")
-        data_file = open('random_binary_data', 'rb')
+        data_file = open(test_fn, 'rb')
         sig = self.gpg.sign_file(data_file, keyid=key.fingerprint,
                                  passphrase='aable', detach=True)
         data_file.close()
@@ -375,7 +383,7 @@ class GPGTestCase(unittest.TestCase):
         self.assertTrue(sig.hash_algo)
         try:
             file = gnupg._make_binary_stream(sig.data, self.gpg.encoding)
-            verified = self.gpg.verify_file(file, 'random_binary_data')
+            verified = self.gpg.verify_file(file, test_fn)
         except UnicodeDecodeError: #happens in Python 2.6
             from io import BytesIO
             verified = self.gpg.verify_file(BytesIO(sig.data))
@@ -384,6 +392,22 @@ class GPGTestCase(unittest.TestCase):
             logger.debug("ver: %r", verified.fingerprint)
         self.assertEqual(key.fingerprint, verified.fingerprint,
                          "Fingerprints must match")
+        # Test in-memory verification
+        data_file = open(test_fn, 'rb')
+        data = data_file.read()
+        data_file.close()
+        fd, fn = tempfile.mkstemp()
+        os.write(fd, sig.data)
+        os.close(fd)
+        try:
+            verified = self.gpg.verify_data(fn, data)
+        finally:
+            os.unlink(fn)
+        if key.fingerprint != verified.fingerprint:
+            logger.debug("key: %r", key.fingerprint)
+            logger.debug("ver: %r", verified.fingerprint)
+        self.assertEqual(key.fingerprint, verified.fingerprint,
+                         "Fingerprints must match")
         logger.debug("test_signature_verification ends")
 
     def test_deletion(self):
@@ -401,7 +425,14 @@ class GPGTestCase(unittest.TestCase):
 
     def test_nogpg(self):
         "Test that absence of gpg is handled correctly"
-        self.assertRaises(ValueError, gnupg.GPG, gnupghome=self.homedir,
+        # We use shell=False on POSIX, shell=True on Windows
+        # and this can lead to one of two possible exceptions
+        # depending on the platform.
+        if os.name == 'posix':
+            exc = OSError
+        else:
+            exc = ValueError
+        self.assertRaises(exc, gnupg.GPG, gnupghome=self.homedir,
                           gpgbinary='frob')
 
     def test_make_args(self):
@@ -416,6 +447,12 @@ class GPGTestCase(unittest.TestCase):
         logger.debug("test_file_encryption_and_decryption begins")
         encfno, encfname = tempfile.mkstemp()
         decfno, decfname = tempfile.mkstemp()
+        if os.name == 'posix':
+            # pick a mode that won't be already in effect via umask
+            mode = os.stat(encfname).st_mode | stat.S_IXUSR
+            os.chmod(encfname, mode)
+            # assume same for decfname
+            os.chmod(decfname, mode)
         # On Windows, if the handles aren't closed, the files can't be deleted
         os.close(encfno)
         os.close(decfno)
@@ -448,6 +485,10 @@ class GPGTestCase(unittest.TestCase):
             self.assertEqual(data, ddata, "Round-trip must work")
         finally:
             for fn in (encfname, decfname):
+                if os.name == 'posix':
+                    # Check that the file wasn't deleted, and that the
+                    # mode bits we set are still in effect
+                    self.assertEqual(os.stat(fn).st_mode, mode)
                 if os.path.exists(fn):
                     os.remove(fn)
         logger.debug("test_file_encryption_and_decryption ends")
@@ -461,6 +502,30 @@ class GPGTestCase(unittest.TestCase):
         self.assertTrue(r)
         self.assertTrue('Vinay Sajip <vinay_sajip@hotmail.com>' in r[0]['uids'])
 
+    def test_quote_with_shell(self):
+        "Test shell quoting with a real shell"
+        if os.name != 'posix': return
+
+        from subprocess import PIPE, Popen
+
+        workdir = tempfile.mkdtemp()
+        try:
+            s = "'\\\"; touch %s/foo #'" % workdir
+            cmd = 'echo %s' % gnupg.shell_quote(s)
+            p = Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE)
+            p.communicate()
+            self.assertEqual(p.returncode, 0)
+            files = os.listdir(workdir)
+            self.assertEqual(files, [])
+            fn = "'ab?'"
+            cmd = 'touch %s/%s' % (workdir, gnupg.shell_quote(fn))
+            p = Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE)
+            p.communicate()
+            self.assertEqual(p.returncode, 0)
+            files = os.listdir(workdir)
+            self.assertEqual(files, ["'ab?'"])
+        finally:
+            shutil.rmtree(workdir)
 
 TEST_GROUPS = {
     'sign' : set(['test_signature_verification']),
@@ -475,7 +540,8 @@ TEST_GROUPS = {
                  'test_search_keys']),
     'import' : set(['test_import_only']),
     'basic' : set(['test_environment', 'test_list_keys_initial',
-                   'test_nogpg', 'test_make_args']),
+                   'test_nogpg', 'test_make_args',
+                   'test_quote_with_shell']),
 }
 
 def suite(args=None):
